"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printableAccount = exports.AccountStates = void 0;
const web3_js_1 = require("@solana/web3.js");
const log_1 = require("../utils/log");
const assert_1 = require("assert");
const deep_diff_1 = require("deep-diff");
const events_1 = __importDefault(require("events"));
class AccountStateTracker {
    constructor() {
        this.states = [];
    }
    add(state) {
        const lastState = this.states.length > 0 ? this.states[this.states.length - 1] : null;
        const accountDiff = lastState == null
            ? undefined
            : (0, deep_diff_1.diff)(lastState.account.pretty(), state.account.pretty());
        this.states.push({ ...state, accountDiff, timestamp: Date.now().valueOf() });
    }
    get relayStates() {
        return this.states.map(({ account, ...rest }) => ({
            account: account.pretty(),
            ...rest,
        }));
    }
}
class AccountStates extends events_1.default {
    constructor(connection, accountProvider) {
        super();
        this.connection = connection;
        this.accountProvider = accountProvider;
        this.states = new Map();
        this._onLog = async (logs, ctx) => {
            const tx = await this.connection.getTransaction(logs.signature, {
                commitment: 'confirmed',
            });
            if (tx == null) {
                (0, log_1.logDebug)(`Could not find transaction ${logs.signature}`);
                return;
            }
            const nonProgramAddresses = tx.transaction.message
                .nonProgramIds()
                .map((x) => x.toBase58());
            for (const key of nonProgramAddresses) {
                this.update(key, ctx.slot);
            }
        };
        this.connection.onLogs('all', this._onLog, 'confirmed');
    }
    async update(address, slot) {
        var _a;
        if (!this.states.has(address)) {
            this.states.set(address, new AccountStateTracker());
        }
        const res = await this.accountProvider.tryResolveAccount(new web3_js_1.PublicKey(address));
        if (res == null)
            return;
        this.add(address, { ...res, slot });
        const states = (_a = this.get(address)) === null || _a === void 0 ? void 0 : _a.relayStates;
        this.emit(`account-changed:${address}`, states);
    }
    add(address, state) {
        const states = this.get(address);
        (0, assert_1.strict)(states != null, 'expected states to be set before adding');
        states.add(state);
    }
    get(address) {
        return this.states.get(address);
    }
    static get instance() {
        (0, assert_1.strict)(AccountStates._instance != null, 'expected AccountStates instance');
        return AccountStates._instance;
    }
    static createInstance(connection, accountProvider) {
        if (AccountStates._instance != null)
            return;
        AccountStates._instance = new AccountStates(connection, accountProvider);
        return AccountStates._instance;
    }
}
exports.AccountStates = AccountStates;
AccountStates._instance = null;
function printableAccount(account) {
    const prettified = {};
    for (const [key, val] of Object.entries(account)) {
        if (val == null)
            continue;
        if (typeof val.pretty === 'function') {
            prettified[key] = val.pretty();
        }
        if (typeof val === 'object') {
            prettified[key] = JSON.stringify(val, null, 2);
        }
    }
    return { ...account, ...prettified };
}
exports.printableAccount = printableAccount;
//# sourceMappingURL=state.js.map